#!/usr/bin/php -q
<?php
/*
 *      (c) Lukas Ziaja <lz@metasec.de>
 */

function zphone_manager_proxy_check_remote() {
    
    if ( !file_exists("/etc/metasec/astmanproxy_servers.conf") ) 
        return false;
    
    $SRV = parse_ini_file ( "/etc/metasec/astmanproxy_servers.conf" , true );
    if ( count($SRV) == 0 )
        return;
    
    
    // get list of current servers logget at astmanproxy
    $CUR_SRV = zphone_manager_list_connected();
    if ( !$CUR_SRV ) 
        return false;
    
    // get hosts
    $_cur_srv = array();
    foreach ( $SRV as $_key => $_srv ) {
        if ( !in_array($_srv["host"], $CUR_SRV ) )
            zphone_manager_proxy_add_srv($_srv["host"],$_srv["user"],$_srv["pass"],$_srv["port"]);
    }
    
    return true;
    
}

function zphone_manager_list_connected() {

    $str_arr = array();
    $socket = @fsockopen("127.0.0.1","5039", $errno, $errstr, 2);
    
    if ( !$socket ) {
        syslog(LOG_WARNING, "ZPHONE ".__FUNCTION__." error: ".$errstr." cannot list proxysessions! astmanproxy seems down.\n ");
        return false;
    }
    
    stream_set_timeout($socket, 0, 10000 );
    $actionID = substr(time(),5,5).substr($_SERVER["REMOTE_ADDR"],strlen($_SERVER["REMOTE_ADDR"])-6,6).rand();

    fputs($socket, "Action: Login\r\n");
    fputs($socket, "UserName: z_mrtg\r\n");
    fputs($socket, "Secret: z_mrtg\r\n");
    fputs($socket, "ActionID: ".$actionID."\r\n\r\n");

    fputs($socket, "ProxyAction: ListSessions\r\n");
    fputs($socket, "ActionID: ".$actionID."\r\n");
    fputs($socket, "\r\n");

    $tst = true;
    $str = "";
    $z=0;
    while ( $tst  ) {

        $line = fread($socket, 8192);
        $line2 = trim($line);

        $str .= $line;

        if ( $z == 20 )
            $tst = false;

        if ( $line2 == "" )
            $z++;
            
        if ( substr_count($line2,"ProxyResponse: Success") ) {
           $tst=false; 
        }

    }
    fputs($socket, "Action: Logoff\r\n\r\n");
    fclose($socket);
    
    
    foreach ( explode("\n", $str ) as $line ) {
        if ( substr_count($line, "ProxyServerSession") ) {
            $_tmp = explode(":", $line);
            $str_arr[] = trim($_tmp[1]);
        }
    }
    
    syslog(LOG_NOTICE, "ZPHONE ".__FUNCTION__." list astmanproxy servers ".count($str_arr)." active!\n ");
    return $str_arr;
    
}
/*
 *  add server on demand to the astmanproxy
 */
function zphone_manager_proxy_add_srv($srv,$user,$pass,$port=5038) {
    
    $socket = @fsockopen("127.0.0.1","5039", $errno, $errstr, 2);
    
    if ( $port == 0 || empty($port) )
        $port=5038;
    
    if ( !$socket ) {
        syslog(LOG_WARNING, "ZPHONE ".__FUNCTION__." error: ".$errstr." cannot add server to proxy! astmanproxy seems down.\n ");
        return false;
    
    }
    
    stream_set_timeout($socket, 0, 10000 );
    $actionID = substr(time(),5,5).substr($_SERVER["REMOTE_ADDR"],strlen($_SERVER["REMOTE_ADDR"])-6,6).rand();

    fputs($socket, "Action: Login\r\n");
    fputs($socket, "UserName: z_mrtg\r\n");
    fputs($socket, "Secret: z_mrtg\r\n\r\n");

    fputs($socket, "ProxyAction: AddServer\r\n");
    fputs($socket, "Server: ".$srv."\r\n");
    fputs($socket, "Port: ".$port."\r\n");
    fputs($socket, "Username: ".$user."\r\n");
    fputs($socket, "Secret: ".$pass."\r\n");
    fputs($socket, "Events: on\r\n\r\n");

    $tst = true;
    $str = "";
    $z=0;
    while ( $tst  ) {

        $line = fread($socket, 8192);
        $line2 = trim($line);

        $str .= $line;

        if ( $z == 20 )
            $tst = false;

        if ( $line2 == "" )
            $z++;
            
        if ( substr_count($line2,"Message: Added ".$srv) ) {
           $tst=false; 
        }

    }
    fputs($socket, "Action: Logoff\r\n\r\n");
    fclose($socket);
    
    //print($str);
    syslog(LOG_WARNING, "ZPHONE ".__FUNCTION__." ".$errstr." added server ".$srv." to proxy!\n ");
    return true;

}

function zphone_manager_conn_ast() {

    $z=1;
    $socket = @fsockopen("127.0.0.1","5038", $errno, $errstr, 2);
    if ( !$socket ) {
        syslog(LOG_WARNING, "ZPHONE ".__FUNCTION__." error: ".$errno." ".$errstr." asterisk not responding?!\n ");
	    return false;
    }

    $actionID = substr(time(),5,5).substr($_SERVER["REMOTE_ADDR"],strlen($_SERVER["REMOTE_ADDR"])-6,6).rand();

    fputs($socket, "Action: Login\r\n");
    fputs($socket, "UserName: z_mrtg\r\n");
    fputs($socket, "Secret: z_mrtg\r\n\r\n");

    fputs($socket, "Action: Ping\r\n");
    fputs($socket, "ActionID: ".$actionID."\r\n\r\n");

    fputs($socket, "Action: Logoff\r\n\r\n");

    $str = "";
    while ( !feof($socket) ) {
        $str .= fread($socket, 8192);
    }

    fclose($socket);

    
    /*
    print("=======================\n");
    print($str);
    print("=======================\n");
    */
    
    if ( !substr_count($str, "Pong" ) ) {
        syslog(LOG_WARNING, "ZPHONE ".__FUNCTION__." error: ".$errstr." asterisk not responding?!\n ");
        return false;
    } else {
        return true;
    }

}

function zphone_manager_conn_proxy() {

    $z      = 1;
    $tst    = true;
    $str    = "";
    $socket = @fsockopen("127.0.0.1","5039", $errno, $errstr, 2);
    
    if ( !$socket  ) {
    
        if ( zphone_manager_conn_ast() ) {
            
	        exec("killall -9 astmanproxy  >/dev/null 2>/dev/null", $out);
	        exec("/etc/init.d/astmanproxy start >/dev/null 2>/dev/null", $out);
	        //print_r($out);
            syslog(LOG_WARNING, "ZPHONE ".__FUNCTION__." error: ".$errstr." no astmanproxy, trying to start\n ");
	        sleep(2);
            return false;
            
        } else {
            // ok... asterisk also down...
            return false;
        }
        
    }

    stream_set_timeout( $socket, 0, 10000 );
    $actionID = substr(time(),5,5).substr($_SERVER["REMOTE_ADDR"],strlen($_SERVER["REMOTE_ADDR"])-6,6).rand();

    fputs($socket, "Action: Login\r\n");
    fputs($socket, "UserName: z_mrtg\r\n");
    fputs($socket, "Secret: z_mrtg\r\n\r\n");

    fputs($socket, "Action: Ping\r\n");
    fputs($socket, "ActionID: ".$actionID."\r\n\r\n");


    while ( $tst  ) {

        $line = fread($socket, 8192);
        $line2 = trim($line);

        if ( substr_count($line2,"Pong") ) {
           $tst = false;
        }
        $str .= $line;

        if ( $z == 20 )
            $tst = false;

        if ( $line2 == "" )
            $z++;

    }
    fputs($socket, "Action: Logoff\r\n\r\n");
    fclose($socket);

    /*
    print("=======================\n");
    print($str);
    print("=======================\n");
    */
    
    if ( !substr_count($str, "Pong" ) ) {
        syslog(LOG_WARNING, "ZPHONE ".__FUNCTION__." error: ".$errstr." Astmanproxy alive but asterisk gone...\n ");
        return false;
    }

    return true;


}

/*
 *  clear lock file on startup
 */
if ( file_exists("/tmp/zphone_astmanproxy_chkd.stop") ) {
    
    print("deleting stop file /tmp/zphone_astmanproxy_chkd.stop \n");
    unlink("/tmp/zphone_astmanproxy_chkd.stop");
    
}


set_time_limit(0);
declare(tick=1);
$pid = pcntl_fork();

if ( $pid == -1 ) {
    die("could not fork");
} elseif ( $pid ) {
    exit();
} else {
    // we are the child!
}

if ( posix_setsid() == -1 )
    die("could not detach form terminal\n");


$test_remote = 58;
/*
 *  loop until we get SIGHUP oder lockfile present
 */
while ( !$z ) {
    zphone_manager_conn_proxy();
    sleep(1);
    
    if ( $test_remote++ == 60 ) {
        zphone_manager_proxy_check_remote();
        $test_remote=0;
    }
    
    if ( file_exists("/tmp/zphone_astmanproxy_chkd.stop") ) {
        syslog(LOG_WARNING, "ZPHONE zphone_astmanproxy_check quiting\n ");
	    break;
    }
    
};
?>